    # def is_free(self, raw):
    #     """ read raw response and detect if the domain is free """
    #     raise NotImplementedError

    # def check_errors(self, raw):
    #     """ read raw response and detect if there are errors """
    #     raise NotImplementedError
        
    # def parse(self, raw):
    #     """ read raw response and fill all data """
    #     raise NotImplementedError
